% Generates and plots filtered and smoothed estimates of states of Extinction - Origination series and its
% interpolates at specified interpolated regular time points using a vector continuous time ZAR model and the
% function VCZstateFilterSmooth.m
clear 
clc

disp('Release version of script ExtOrigBookModelInterp')

% Model structure as in the book Table 8.3 but with slightly improved coefficients
% ********************************************************************************
load ExtOrigBookModelWeb % yRef m p kappa obsnoise VCZARcoeff VCZARSig NoiseSig xregcoeff Strucoeff parind struc


load lgeps.txt  % irregular time series of logarithms of extinctions of genera
load lgops.txt  % irregular time series of logarithms of originations of genera

% These original series are used only for final plotting with the filtered and
% smoothed series, which are computed from the structure yRef saved after model
% estimation. The series will be shortened as used in model estimation, and their means 
% then computed for use in restoring the means of the filtered and smoothed series.
% This is because the series were mean corrected (to improve conditioning) before model
% estimation, which carried through to yRef. 
% The original series time vector is time before present (tbp), and the series
% values correspond to this, being presented in reverse time order. So for model 
% estimation the time vector was subtracted from 545, to give values that 
% increase from past to present though still in reverse sequence in the vector.
% This sequence is rectified in yRef as one of the first steps of model estimation is to
% order the series values for increasing time. The trend regressors are still in the
% original (tbp) order, so the index column of yRef must be used to obtain the correctly
% ordered trend, when it is used for series correction or restoration.

n1=length(lgeps(:,2));
n2=length(lgops(:,2));

% Use slightly shorter subseries as in published source

sub1=[1:(n1-3)];
sub2=[1:(n2-3)];

lgeps=lgeps(sub1,:);
lgops=lgops(sub2,:);

lgeps(:,1)=545-lgeps(:,1);  % reverse from time before present 
lgops(:,1)=545-lgops(:,1);

mntim=(lgeps(:,1)+lgops(:,1))/2;
 
lgeps(:,1)=mntim;  % change to a common  mid point time vector
lgops(:,1)=mntim;


n1=length(lgeps(:,2))
n2=length(lgops(:,2))
N=n1+n2;

mlgeps=mn(1);mlgops=mn(2); % mean correct for filtering and smoothing
lgeps(:,2) = lgeps(:,2) - mlgeps;
lgops(:,2) = lgops(:,2) - mlgops;

y={lgeps,lgops};

yRef=model{14};

obstim=yRef(:,3);
obsval=yRef(:,4);
obslen=length(obstim);

[n,null]=size(yRef);

%insert extra time points for estimated states with missing observations

tfirst=yRef(1,3);
tlast=yRef(n,3);
nxtra=200;
nxtra=nxtra-1;
txtra=tfirst+(tlast-tfirst)*[1:nxtra]/(nxtra+1);
yRefxtra=zeros(nxtra*2,4);
yRefxtra(:,1)=[ones(nxtra,1);2*ones(nxtra,1)];
yRefxtra(:,2)=0;
yRefxtra(:,3)=[txtra';txtra'];
yRefxtra(:,4)=NaN;

% extend and redefine yRef by sorting over time
yRef=[yRef;yRefxtra];
[null,sortIndex] = sort(yRef(:,3));
yRef = yRef(sortIndex,:);

subtim=find(yRef(:,2)~=0);
ser1ind=find(yRef(:,1)==1);
n1=length(ser1ind);
ser2ind=find(yRef(:,1)==2);
n2=length(ser2ind);

Nobs=size(yRef,1);
yRef(ser1ind,2)=[1:n1]';
yRef(ser2ind,2)=[1:n2]';

% extend and redefine regressors
con1=ones(n1,1);
trend1=yRef(ser1ind,3);
con2=ones(n2,1);
trend2=yRef(ser2ind,3);

xreg1=[con1,trend1];
xreg2=[con2,trend2];
xreg={xreg1,xreg2};

xregnum=zeros(1,m);
xregind=sum(size(xreg))>0;
if xregind==1
    for i=1:m
        xregnum(i)=size(xreg{i},2);
    end
end 

% get model parameters
VCZARcoeff=model{1}
VCZARSig=model{2}
NoiseSig=model{3};
xregcoeff=model{4};
Strucoeff=model{6}

%carry out state filtering and smoothing ***************************************************************** 

[statfil,statsmo,trend,Vstatfil,Vstatsmo,cTranMat] =...
    VCZstateFilterSmooth(yRef,m,p,kappa,obsnoise,xreg,xregnum,VCZARcoeff,VCZARSig,NoiseSig,xregcoeff,Strucoeff,parind,struc);

% extract series information from states

lgeps(:,2) = lgeps(:,2) + mlgeps; % restore means of original series
lgops(:,2) = lgops(:,2) + mlgops;

lgeps(:,1)=545-lgeps(:,1);  % revert to from time before present 
lgops(:,1)=545-lgops(:,1);  % for original series

time1=545 - yRef(ser1ind,3); % revert to time before present
% construct filtered and smoothed series
ser1=yRef(ser1ind,4)+mlgeps;
filser1=statfil(1,ser1ind)+trend(ser1ind)+mlgeps;
smoser1=statsmo(1,ser1ind)+trend(ser1ind)+mlgeps;

time2=545 - yRef(ser2ind,3); % revert to time before present
ser2=yRef(ser2ind,4)+mlgops;
filser2=statfil(2,ser2ind)+trend(ser2ind)+mlgops;
smoser2=statsmo(2,ser2ind)+trend(ser2ind)+mlgops;


SDfilser1=sqrt(squeeze(Vstatfil(1,1,ser1ind)))';
SDsmoser1=sqrt(squeeze(Vstatsmo(1,1,ser1ind)))';


SDfilser2=sqrt(squeeze(Vstatfil(2,2,ser2ind)))';
SDsmoser2=sqrt(squeeze(Vstatsmo(2,2,ser2ind)))';

% plot filtered and smoothed series, ,restoring mean levels

figure(1)
plot(time1,filser1,'-b','linewidth',2)
hold on
plot(time1,filser1+2*SDfilser1,'-r','linewidth',1)
plot(time1,filser1-2*SDfilser1,'-r','linewidth',1)
plot(lgeps(:,1),lgeps(:,2),'-*k','linewidth',1)
title('Filtered estimate of transformed extinctions','Fontsize',15)
hold off
set(gca, 'fontsize',12)
set(gca,'PlotBoxAspectRatio',[6,2,1])
xlabel('Time in Myr before present','Fontsize', 15)
axis([-10,545,0,5])
set(gca,'XDir','reverse')




figure(2)
plot(time1,smoser1,'-b','linewidth',2)
hold on
plot(time1,smoser1+2*SDsmoser1,'-r','linewidth',1)
plot(time1,smoser1-2*SDsmoser1,'-r','linewidth',1)
plot(lgeps(:,1),lgeps(:,2),'-*k','linewidth',1)
title('Smoothed estimate of transformed extinctions','Fontsize',15)
hold off
set(gca, 'fontsize',12)
set(gca,'PlotBoxAspectRatio',[6,2,1])
xlabel('Time in Myr before present','Fontsize', 15)
axis([-10,545,0,5])
set(gca,'XDir','reverse')


figure(3)
plot(time2,filser2,'-b','linewidth',2)
hold on
plot(time1,filser2+2*SDfilser2,'-r','linewidth',1)
plot(time1,filser2-2*SDfilser2,'-r','linewidth',1)
plot(lgops(:,1),lgops(:,2),'-*k','linewidth',1)
title('Filtered estimate of transformed originations','Fontsize',15)
hold off
set(gca, 'fontsize',12)
set(gca,'PlotBoxAspectRatio',[6,2,1])
xlabel('Time in Myr before present','Fontsize', 15)
axis([-10,545,1,5])
set(gca,'XDir','reverse')


figure(4)
plot(time2,smoser2,'-b','linewidth',2)
hold on
plot(time1,smoser2+2*SDsmoser2,'-r','linewidth',1)
plot(time1,smoser2-2*SDsmoser2,'-r','linewidth',1)
plot(lgops(:,1),lgops(:,2),'-*k','linewidth',1)
title('Smoothed estimate of transformed originations','Fontsize',15)
hold off
set(gca, 'fontsize',12)
set(gca,'PlotBoxAspectRatio',[6,2,1])
xlabel('Time in Myr before present','Fontsize', 15)
axis([-10,545,1,5])
set(gca,'XDir','reverse')


figure(5)
plot(time2,filser2,'-b','linewidth',2)
hold on
plot(time1,filser1,'-r','linewidth',2)
title('Filtered estimates of transformed extinctions and  originations','Fontsize',15)
hold off
set(gca, 'fontsize',12)
set(gca,'PlotBoxAspectRatio',[6,2,1])
xlabel('Time in Myr before present','Fontsize', 15)
axis([-10,545,1,4])
set(gca,'XDir','reverse')


figure(6)
plot(time2,smoser2,'-b','linewidth',2)
hold on
plot(time1,smoser1,'-r','linewidth',2)
title('Smoothed estimates of transformed extinctions (red) and  originations (blue)','Fontsize',15)
hold off
set(gca, 'fontsize',12)
set(gca,'PlotBoxAspectRatio',[6,2,1])
xlabel('Time in Myr before present','Fontsize', 15)
axis([-10,545,1,4])
set(gca,'XDir','reverse')


return
